/****************************************************************************
 *  Copyright (C) 2018 RoboMaster.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

#include "bsp_pwm.h"

/**
  * @brief  start pwm output
  * @param  None
  * @retval None
  */
void pwm_init(void)
{
    HAL_TIM_PWM_Start(&htim1, TIM_CHANNEL_1); // start pwm output
    HAL_TIM_PWM_Start(&htim1, TIM_CHANNEL_2);
    HAL_TIM_PWM_Start(&htim1, TIM_CHANNEL_3);
    HAL_TIM_PWM_Start(&htim1, TIM_CHANNEL_4);
}

/**
  * @brief  sets PWM duty cycle (= speed) for a particular channel
  * @param  Timer, channel (1 to 4), duty cycle (between 0 and 1)
  * @retval None
  */
void PWM_SetDuty(TIM_HandleTypeDef *tim,uint32_t tim_channel,float duty){
	duty = duty/10 + 0.10f; //rescale before send
	switch(tim_channel){	
		case TIM_CHANNEL_1: tim->Instance->CCR1 = (PWM_RESOLUTION*duty) - 1;break;
		case TIM_CHANNEL_2: tim->Instance->CCR2 = (PWM_RESOLUTION*duty) - 1;break;
		case TIM_CHANNEL_3: tim->Instance->CCR3 = (PWM_RESOLUTION*duty) - 1;break;
		case TIM_CHANNEL_4: tim->Instance->CCR4 = (PWM_RESOLUTION*duty) - 1;break;
	}
}

/**
  * @brief  sets PWM duty cycle (= speed) for a all channels
  * @param  Timer, duty cycles (between 0 and 1)
  * @retval None
  */
void PWM_SetAllDuty(TIM_HandleTypeDef *tim, float duty_ch1, float duty_ch2, float duty_ch3, float duty_ch4){
	duty_ch1 = duty_ch1/10 + 0.10f; //rescale before send
	duty_ch2 = duty_ch2/10 + 0.10f;
	duty_ch3 = duty_ch3/10 + 0.10f;
	duty_ch4 = duty_ch4/10 + 0.10f;
	tim->Instance->CCR1 = (PWM_RESOLUTION*duty_ch1) - 1;
	tim->Instance->CCR2 = (PWM_RESOLUTION*duty_ch2) - 1;
	tim->Instance->CCR3 = (PWM_RESOLUTION*duty_ch3) - 1;
	tim->Instance->CCR4 = (PWM_RESOLUTION*duty_ch4) - 1;
	
}

/**
  * @brief  scales all PWM duty cycles between 0 and 1
  * @param  Timer
  * @retval None
  */
void PWM_ScaleAll(TIM_HandleTypeDef *tim){ //il faudrait jouer sur l'allumage des ports d'alimentation des snails (voir Nathan pour plus de dtails)
	HAL_GPIO_WritePin(GPIOH, POWER1_CTRL_Pin, GPIO_PIN_RESET); // switch off 24v power
	PWM_SetAllDuty(&htim1,1,1,1,1);
	HAL_Delay(10);
	
	HAL_GPIO_WritePin(GPIOH, POWER1_CTRL_Pin, GPIO_PIN_SET); // switch on 24v power
	HAL_Delay(3500);
	
	PWM_SetAllDuty(&htim1,0,0,0,0);
	HAL_Delay(500);
	
}
